﻿/* 【聊天记录管理器】：功能说明*
 * 
 * ==== 关键词解析 ====
 * 
 * [聊天]：
 * 先期只要包括：“短信”，“飞信”的聊天记录。
 * 后期可能会增加QQ的聊天记录
 * 
 * [管理]：
 * 1.短信图片显示
 * -2.短信文本美化显示
 * -3.聊天记录整合（飞信和短信的整合）
 * -4.聊天记录整理：包括“排序[时间]”，“删除”，等等操纵
 * -5.聊天记录摘取功能：某天日期的全聊天记录（针对飞信，短信）
 * 
 * [其它附加功能]：
 * 前期：全手动打开浏览记录文件（一般为纯文本格式[*.TXT][*.CSV]）
 * 后期：可能加入记录自动扫描，保存，解析功能。搜索默认飞信目录，QQ目录等等。
 * 
 * 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
namespace 聊天记录管理器
{
    public partial class frmChattingHistoryManager : Form
    {
        
        #region ========== 自定义变量区 ==========

        Encoding UserEncoding = Encoding.UTF8;
        
        /// <summary>
        /// 我的自定义分隔符。用于自定义保存数据。
        /// </summary>
        string[] MyFormats = {"\\g","\\y"};

        //static  string MyNickName = "时空漫游者";

        //【信息类型】定义：Nokia短信有sms,mms。飞信，QQ
        static string MyTypeNokia = "Nokia短信";
        static string MyTypeFeiXin = "飞信";
        static string MyTypeQQ = "QQ";

        //记录信息类型的总数
        int MyTypeCount = 0;

        //记录当前鼠标下的坐标，用于换算鼠标下的项的索引。
        Point poiItemIndex = new Point();

        //记录一次性读取的文件个数。
        int intFileCount = 0;

        //TODO：此处的快照过多可能会和彼此状态不同步，会出现问题，暂时不解决。

        //保存ListView控件的初始状态。
        List<ListViewItem> lstListViewItems = new List<ListViewItem>();

        //保存当前列表框的状态。
        List<ListViewItem> lstListVIsame = new List<ListViewItem>();

        //保存搜索动作之前的当前ListView控件状态。
        List<ListViewItem> lstItemsSearch = new List<ListViewItem>();

        //保存检测飞信前的一个快照。
        List<ListViewItem> lstFeiXinBackup = new List<ListViewItem>();

        //保存进入“多种信息整理模式”前的状态。
        List<ListViewItem> lstMultiArrangeBackup = new List<ListViewItem>();

        //标记Nokia短信是否仅仅有发送的短信或者接收的短信，以此来判定“保存对话框”里面的提示内容。
        bool isThereSender = false;
        bool isThereReciver = false;
        #endregion


        #region /********************** Read 代码区域 ******************************/

        /// <summary>
        ///之所以使用类而不是使用结构，因为此类的元素都会被存储为引用类型！
        ///[参考资料]:《C#高级编程第六版》P1021
        /// </summary>
        private class NokiaSMS
        {
            /// <summary>
            /// 标识是[短信记录]还是[彩信记录]
            /// </summary>
            public string Type;
            public string FromNumber;
            public string ToNumber;

            public string DateTime;
            public string Title;
            /// <summary>
            /// 信息的内容
            /// </summary>
            public string Content;

        }

        /// <summary>
        /// 读取文件的总入口函数。
        /// </summary>
        /// <param name="ChattingHistoryPath"></param>
        /// <returns>返回读取是否成功的Boolean值</returns>
        private void YERead(string ChattingHistoryPath)
        {
            //再次设置Sorting属性为None，这样就能避免了“清除”之后，再次读取文件会变超慢的问题！
            lsvMessageShower.Sorting = SortOrder.None;

            if ( lstFeiXinBackup.Count != 0 )
            {
                lsvMessageShower.Items.Clear();
                lsvMessageShower.Items.AddRange(lstFeiXinBackup.ToArray());
                //马上初始化此快照。
                lstFeiXinBackup.Clear();
            }

            string strContents = File.ReadAllText(ChattingHistoryPath , UserEncoding);

            if ( YEReadMyData(strContents) )
                ;
            else if ( YEReadNokiaMessages(strContents) )
                ;
            else if ( YEReadFeiXin(strContents) )
                ;
            else if ( YEReadQQMessages(strContents) )
                ;
            else
                MessageBox.Show("不是正确的[聊天信息]导出文件！\n\n请重新选择……","-_-。sorry！" , MessageBoxButtons.OK , MessageBoxIcon.Error);


            //当要读取的文件只有一个的时候，才开始执行相应函数。没执行一次计数器减1。
            if ( intFileCount-- == 1 )
            {
                //重置此变量使得每次的状态起点为当前列表框的信息加上新读取的信息。
                lstListViewItems.Clear();

                YEAutoCheckFeiXin();//Foreach循环2

                YERefreshTheForm();
            }

        }

        /// <summary>
        /// 读取自己定义存储的信息。
        /// </summary>
        /// <param name="Contents"></param>
        /// <returns></returns>
        private bool YEReadMyData(string Contents)
        {
            try
            {
                //检测是否含有特定的分隔符。
                if ( !( Contents.Contains(MyFormats[ 0 ]) && Contents.Contains(MyFormats[ 1 ]) ) )
                    return false;

                List<string> items = new List<string>();
                items.AddRange(Contents.Split(MyFormats , StringSplitOptions.None));
                ListViewItem a = null;
                int i=0;
                foreach ( string var in items )
                {
                    if ( i%5 == 0 )
                        a = new ListViewItem(var);
                    else
                        a.SubItems.Add(var);
                    //曾经碰到i=20的时候，出现异常，所以后面加上i % 5 != 0 判断。
                    if ( i != 0 && i % 4 == 0 && i % 5 != 0 )
                    {
                        lsvMessageShower.Items.Add(a);
                    }
                    i++;

                }
                return true;
            }
            catch(Exception ex)
            {
                Debug.Print(ex.ToString());
                return false;
            }
        }

        private bool YEReadFeiXin(string Contents)
        {
            //只有飞信的导出才有【\t】，没有的话直接退出。
            if ( !Contents.Contains("\t(") )
                return false;
            try
            {
                #region 局部变量声明区

                List<string> feixin = new List<string>();
                List<string> feixinTEMP = new List<string>();
                
                string Separators1 =  "\t(";
                string Separators2 =  "\r\n";
                if ( Contents.Contains("\r\r\n") )
                    Contents = Contents.Replace("\r\r\n" , "\r\n");//把飞信2010版本的导出格式转换为之前的格式。o(︶︿︶)o 唉！
                string Separators3 =  ")说:";

                //定位当前【\t(】制表符的位置
                int idxTab1 = 0;

                //定位下一个Tab索引。
                int idxTab2 = 0;

                //定位当前制表符前面的该符号。
                int idxNextLine1 = 0;

                //定位从下个制表符开始的上一个【\r\n】符号所在位置
                int idxNextLine2 = 0;

                //定位【)说:  】的所在位置
                int idx3 = 0;

                //聊天的昵称
                string name = "";
                //发送时间
                string time = "";
                //聊天内容
                string message = "";

                bool isEnd = false;

                #endregion

                while ( !isEnd )
                {

                    #region 分离各部分信息
                    /* 
                     * 枫之梦23\t(2010-10-19 23:31:00)说:  好了，你快些完成你今天的任务睡觉吧！\r\n我先睡咯！\r\n
                     * 枫之梦23\t(2010-10-19 23:31:00)说:  好了，你快些完成你今天的任务睡觉吧！\r\n我先睡咯！\r\n
                     * 枫之梦23\t(2010-10-19 23:31:00)说:  好了，你快些完成你今天的任务睡觉吧！\r\n我先睡咯！
                     */
                    //定位第一个[\t(]符号的位置。
                    idxTab1 = Contents.IndexOf(Separators1 , idxTab1 + 1);

                    //定位idxTab1前面的【\r\n】符号的位置。
                    idxNextLine1 = Contents.LastIndexOf(Separators2 , idxTab1);
                    if ( idxNextLine1 == -1 )
                        name = Contents.Substring(0, idxTab1);
                    else
                        name = Contents.Substring(idxNextLine1 + Separators2.Length , idxTab1 - idxNextLine1 - Separators2.Length);

                    //定位下一个【\t(】符号的位置。
                    idxTab2 = Contents.IndexOf(Separators1 , idxTab1 + 1);

                    //当不存在下一个【\t(】符号的时候，即没有下一行信息了。
                    if ( idxTab2 == -1 )
                    {
                        isEnd = true;
                        idxNextLine2 = Contents.Length;//除去后面占两个位的/r/n。
                    }
                    else
                    {
                        //定位从下个制表符开始的上一个【\r\n】符号所在位置
                        idxNextLine2 = Contents.LastIndexOf(Separators2 , idxTab2);
                    }

                    //定位【)说:  】的所在位置
                    idx3 = Contents.IndexOf(Separators3 , idxTab1);

                    time = Contents.Substring(idxTab1 + 2 , idx3 - idxTab1 - 2);

                    message = Contents.Substring(idx3 + 5 , idxNextLine2 - idx3 - 5);

                    #endregion

                    //输出显示。
                    int lastIndex = lsvMessageShower.Items.Count;
                    lsvMessageShower.Items.Add("飞信");
                    if ( name == MyNickName.Text )
                    {
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add("");
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add(name);
                    }
                    else
                    {
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add(name);
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add("");
                    }
                        
                    lsvMessageShower.Items[ lastIndex ].SubItems.Add(time);
                    lsvMessageShower.Items[ lastIndex ].SubItems.Add(message);

                }
                return true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return false;
            }

        }

        /// <summary>
        /// 开始后读取Nokia导出的.csv格式的短信备份记录。
        /// </summary>
        /// <param name="strContents">包含要分离解析的所有文本内容。</param>
        /// <param name="Type">要读取的类型。短信("sms")还是彩信("mms")或者自动("auto")。</param>
        private bool YEReadNokiaMessages(string strContents)
        {
            //只有内容中包含Nokia短信特有的两个符号才继续执行。
            if ( !( strContents.Contains("sms") || strContents.Contains("mms") ) )
                return false;

            #region 初始化必须的局部变量

            string Type =  "sms";

            //【注意！！】因为NokiaSMS声明为类“引用”类型！所以必须单独初始化！否则所有数据都只引用一个数据内容！
            NokiaSMS NokiaSMSTEMP = null;

            //确定每个Type的位置。
            int indexType = 0;

            //标记这条Type的结束位置。
            int indexP = 0;

            //被完整截取出来的一条短信。
            string message = null;

            //分离MMS所需的定位子字符串。
            string[] MMS_Separators = new string[] { ",\"" , "\",\"" , "\"," };
            string[] SMS_Separators = new string[] { "," };

            //临时存储一条短信分离出的各种信息。
            List<string> contentTEMP = new List<string>();

            #endregion

            try
            {

                do
                {
                    #region 分离出一条完整的短信

                    Type = "sms";
                    if ( strContents.Contains("mms") )
                        Type = "mms";//首先把彩信先分离。

                    indexType = strContents.IndexOf(Type);
                    if ( indexType == -1 )
                        throw new Exception();

                    indexP = 0;

                    //获取下一个（假如存在的话）短信（或彩信）记录的索引。
                    int indexNextMMS = strContents.IndexOf("mms" , indexType + 3);
                    int indexNextSMS = strContents.IndexOf("sms" , indexType + 3);

                    //都存在下一条记录的情况下，必须选索引最小(即真正的紧接着这条MMS的下一条记录的索引)的。
                    if ( indexNextSMS != -1 && indexNextMMS != -1 )
                        indexP = ( indexNextMMS < indexNextSMS ) ? indexNextMMS : indexNextSMS;
                    //存在下一条MMS记录的时候，只能选该索引了。
                    else if ( indexNextMMS != -1 && indexNextSMS == -1 )
                        indexP = indexNextMMS;
                    //存在下一条SMS记录的时候，只能选该索引了。
                    else if ( indexNextSMS != -1 && indexNextMMS == -1 )
                        indexP = indexNextSMS;
                    //假如都不存在的话，则把最后的文档长度复制给它。
                    else
                        indexP = strContents.Length - 1;

                    message = strContents.Substring(indexType , indexP - indexType);

                    #endregion

                    //将当前分离出来的记录清除。
                    strContents = strContents.Remove(indexType , indexP - indexType);

                    //【注意！！】每次循环在这实例化一个NokiaSMS才能保证每个实例引用不同的数据！
                    NokiaSMSTEMP = new NokiaSMS();
                    contentTEMP.Clear();

                    if ( Type == "mms" )
                    {
                        #region 分离一条“彩信”的各部分内容。

                        //【Nokia短信导出格式】
                        //mms,"+8615079154016","+8613576052330","","2010.09.11 12:47","Title","\n既然胃"，

                        contentTEMP.AddRange(message.Split(MMS_Separators , StringSplitOptions.None));
                        NokiaSMSTEMP.Type = contentTEMP[ 0 ];
                        NokiaSMSTEMP.FromNumber = contentTEMP[ 1 ];
                        NokiaSMSTEMP.ToNumber = contentTEMP[ 2 ];
                        NokiaSMSTEMP.DateTime = contentTEMP[ 4 ];
                        NokiaSMSTEMP.Title = contentTEMP[ 5 ];
                        NokiaSMSTEMP.Content = contentTEMP[ 6 ].TrimEnd('\n').Trim('"');

                        //由于没有单独显示Title的列，所以直接合并到内容列上面。
                        if ( NokiaSMSTEMP.Title != "" )
                            NokiaSMSTEMP.Content = "[" + NokiaSMSTEMP.Title + "]：" + NokiaSMSTEMP.Content;
                        #endregion
                    }
                    else
                    {
                        #region 分离一条“短信”的各部分内容。

                        //【Nokia短信导出格式】
                        //sms,deliver,"+861252015296005188","","","2010.09.22 14:59","","谢谢,你那下雨吗？"

                        contentTEMP.AddRange(message.Split(SMS_Separators , StringSplitOptions.None));

                        //把最后的短信内容部分通通去掉。因为会出现（,）干扰，致使信息分离不准确。
                        contentTEMP.RemoveRange(7 , contentTEMP.Count - 7);

                        //唯有（,"）这个标识每条短信都是固定在最后一个位置
                        contentTEMP.Add(message.Substring(message.LastIndexOf(",\"") + 1));


                        //索引标识：
                        //1.标记：deliver（接收）或submit（发送）
                        //2.标识：手机号码。
                        //5.标识：时间
                        //7.标识：短信内容

                        //去掉前后的引号。

                        NokiaSMSTEMP.Type = contentTEMP[ 0 ].Trim('"');
                        NokiaSMSTEMP.FromNumber = contentTEMP[ 2 ].Trim('"');
                        NokiaSMSTEMP.ToNumber = contentTEMP[ 3 ].Trim('"');
                        NokiaSMSTEMP.DateTime = contentTEMP[ 5 ].Trim('"');
                        NokiaSMSTEMP.Content = contentTEMP[ 7 ].TrimEnd('\n').Trim('"');

                        #endregion
                    }

                    YEShowNokiaSMS(NokiaSMSTEMP , Type);

                } while ( strContents.Contains("sms") || strContents.Contains("mms"));

                return true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return false;
            }
        }

        /// <summary>
        /// 将Record信息显示到ListView控件上。
        /// </summary>
        /// <param name="SMS"></param>
        private void YEShowNokiaSMS(NokiaSMS SMS , string Type)
        {
            try
            {
                //确定添加到列表框的位置。
                int lastIndex = lsvMessageShower.Items.Count;
                
                //添加原始信息。
                lsvMessageShower.Items.Add(Type);
                lsvMessageShower.Items[ lastIndex ].SubItems.Add(SMS.FromNumber);
                lsvMessageShower.Items[ lastIndex ].SubItems.Add(SMS.ToNumber);
                lsvMessageShower.Items[ lastIndex ].SubItems.Add(SMS.DateTime);
                lsvMessageShower.Items[ lastIndex ].SubItems.Add(SMS.Content);
                if ( !isThereReciver && SMS.FromNumber != "" )
                    isThereReciver = true;
                if ( !isThereSender && SMS.ToNumber != "" )
                    isThereSender = true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return;
            }
        }

        private bool YEReadQQMessages(string Contents)
        {
            //只有当文件中存在QQ导出文件特有的字符串，才开始执行。
            if (! Contents.Contains("消息对象") )
                return false;
            try
            {
                List<string> messages = new List<string>();
                messages.AddRange(Contents.Split(new string[] { "\r\n" } , StringSplitOptions.RemoveEmptyEntries));

                //获得“消息对象”出现的索引，用于提取网名。
                int last = messages.FindIndex(0 , 10 , YEFindQQName);
                string sender = messages[ last ].Substring(5);

                //将后面的QQ号码去掉(存在的话)，只留下网名（或备注名）
                int j = sender.LastIndexOf('(');
                if ( j != -1 )
                    sender = sender.Remove(j);

                //把前面9行清除掉。
                messages.RemoveRange(0 , last + 2);

                //从第10行开始就是真正的聊天内容。
                int index = 0;
                int indexNext = 0;
                int indexTEMP = 0;

                string name = "";
                string time = "";
                string mess = "";

                do
                {
                    #region 循环分离每条聊天信息

                    index = messages.FindIndex(YEFindQQNumber);
                    //必须对值判断，以防文件末尾的空格造成异常！
                    if ( index == -1 )
                        break;
                    time = messages[ index ];

                    indexTEMP = time.LastIndexOf(' ');

                    //获得发送者。
                    name = time.Substring(indexTEMP + 1);

                    //获得发送时间。
                    time = time.Remove(indexTEMP).Trim();
                    time = DateTime.Parse(time).ToString("yyyy-MM-dd HH:mm:ss");

                    //当indexNext为-1的时候到达文件末尾。
                    indexNext = messages.FindIndex(1 , YEFindQQNumber);
                    if ( indexNext == -1 )
                        indexNext = 2;

                    //获得发送信息。
                    mess = string.Empty;
                    for ( int i = index + 1 ; i < indexNext ; i++ )
                        mess = mess + messages[ i ];


                    //最后删除已经读取的信息，减少以后检索Count。
                    messages.RemoveRange(index , indexNext - index);

                    //将信息显示到控件
                    int lastIndex = lsvMessageShower.Items.Count;
                    lsvMessageShower.Items.Add("QQ");
                    if ( name == MyNickName.Text )
                    {
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add("");
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add(name);
                    }
                    else
                    {
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add(name);
                        lsvMessageShower.Items[ lastIndex ].SubItems.Add("");
                    }
                    lsvMessageShower.Items[ lastIndex ].SubItems.Add(time);
                    lsvMessageShower.Items[ lastIndex ].SubItems.Add(mess);

                    #endregion


                } while ( messages.Count != 0 );
                return true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return false;
            }
        
        }

        /// <summary>
        /// 通过严格定位每个‘-’符号，‘：’符合和‘_’符合的位置来判断是否为首行QQ信息。
        /// </summary>
        /// <param name="s"></param>
        /// <returns></returns>
        private static bool YEFindQQNumber(string s)
        {
            //【QQ2010】导出格式："2010-9-18 [2010/10/17]23:22:50 ぃ记忆、定格"	string
            //【QQ2009】导出格式："2010-04-22 18:36:33 秋权"
            if ( (s.Contains('-')||s.Contains('/')) && s.Contains(':') )
            {
                int a1 = s.IndexOf('-');
                if ( a1 == -1 )
                    a1 = s.IndexOf('/');
                int a2 = s.LastIndexOf('-');
                if ( a2 == -1 )
                    a2 = s.LastIndexOf('/');
                int b1 = s.IndexOf(':');
                int b2 = s.LastIndexOf(':');
                if ((a1==4)&&((a2-a1)<=3)&&((b2-b1)<=3))
                    return true;
                else
                    return false;
            }
            else
                return false;
                
        }		

        /// <summary>
        /// 通过测定“消息对象”的位置来获取网名。
        /// </summary>
        /// <param name="s"></param>
        /// <returns></returns>
        private static bool YEFindQQName(string s)
        {
            if ( ( s.Length > 5 ) && s.Contains("消息对象"))
            {
                return true;
            }
            else
            {
                return false;
            }

        }

        #endregion


        #region /********************** Save 代码区域 ******************************/

        /// <summary>
        /// 将新顺序保存到文件。
        /// </summary>
        /// <param name="FilePath"></param>
        private bool YESaveNokiaMessages(string FilePath)
        {
            try
            {
                StringBuilder strNewContents = new StringBuilder();
                foreach ( ListViewItem tmp in lsvMessageShower.Items )
                {
                    if ( tmp.SubItems[ 0 ].Text == "mms" )
                    {

                        //【Nokia短信导出格式】
                        //mms,"+8615079154016","+8613576052330","","2010.09.11 12:47","","\n既然胃"，
                        strNewContents.Append("mms,"
                            + "\"" + tmp.SubItems[ 1 ].Text + "\","
                            + "\"" + tmp.SubItems[ 2 ].Text + "\","
                            + "\"\","
                            + "\"" + tmp.SubItems[ 3 ].Text + "\","
                            + "\"\","
                            + "\"" + tmp.SubItems[ 4 ].Text + "\",\n");
                    }
                    else
                    {
                        //【Nokia短信导出格式】
                        //sms,deliver,"+861252015296005188","","","2010.09.22 14:59","","谢谢,你那下雨吗？"
                        string smsHead = "";
                        if ( tmp.SubItems[ 1 ].Text != "" )
                            smsHead = "sms,deliver,";
                        else
                            smsHead = "sms,submit,";

                        strNewContents.Append(smsHead
                            + "\"" + tmp.SubItems[ 1 ].Text + "\","
                            + "\"" + tmp.SubItems[ 2 ].Text + "\","
                            + "\"\","
                            + "\"" + tmp.SubItems[ 3 ].Text + "\","
                            + "\"\","
                            + "\"" + tmp.SubItems[ 4 ].Text + "\"\n");
                    }
                }

                File.WriteAllText(FilePath , strNewContents.ToString() , Encoding.UTF8);
                return true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return false;
            }
        }

        /// <summary>
        /// 三种信息的合并后的信息输出函数！
        /// </summary>
        private bool YESaveNewList(string FilePath)
        {
            try
            {
                //使用【\g】分隔列，使用【\y】分隔行。
                StringBuilder h = new StringBuilder();
                foreach ( ListViewItem item in lsvMessageShower.Items )
                {
                    h.Append(item.SubItems[ 0 ].Text);
                    h.Append(MyFormats[ 0 ]);
                    h.Append(item.SubItems[ 1 ].Text);
                    h.Append(MyFormats[ 0 ]);
                    h.Append(item.SubItems[ 2 ].Text);
                    h.Append(MyFormats[ 0 ]);
                    h.Append(item.SubItems[ 3 ].Text);
                    h.Append(MyFormats[ 0 ]);
                    h.Append(item.SubItems[ 4 ].Text);
                    h.Append(MyFormats[ 1 ]);
                }
                File.WriteAllText(FilePath , h.ToString() , Encoding.UTF8);

                return true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return false;
            }
        }

        /// <summary>
        /// 输出保存聊天记录美化后的文本。
        /// </summary>
        /// <param name="FilePath"></param>
        private bool YESaveBeautifulText(string FilePath)
        {
            try
            {
                List<string> strBeautifulText = new List<string>();
                strBeautifulText.Add("╔--------------------------╗");
                strBeautifulText.Add("┆╰(￣▽￣)╮By：AsionTang ┆");
                strBeautifulText.Add("╠--------------------------╩---------------------------------╗");
                strBeautifulText.Add("╠-------------------------------------------------------------╝");
                string star = "★";
                List<string> phoneMenber = new List<string>();
                string startTime = "";
                string endTime = "";

                #region 循环区域
                foreach ( ListViewItem tmp in lsvMessageShower.Items )
                {
                    if ( startTime == "" )
                        startTime = tmp.SubItems[ 3 ].Text;
                    if ( tmp.SubItems[ 3 ].Text.CompareTo(startTime) > 0 )
                    {
                        endTime = tmp.SubItems[ 3 ].Text;
                    }


                    //deliver（接收）或submit（发送）
                    if ( tmp.SubItems[ 1 ].Text != "" )
                    {

                        if ( !phoneMenber.Contains(tmp.SubItems[ 1 ].Text) )
                        {
                            phoneMenber.Add(tmp.SubItems[ 1 ].Text);
                        }
                        //是接收到的信息 就显示在左边
                        strBeautifulText.Add(star + "FROM:℡﹛" + tmp.SubItems[ 1 ].Text + "﹜");
                        if ( star == "★" )
                            star = "☆";
                        else
                            star = "★";
                        strBeautifulText.Add("┆————————————————————");
                        string content = tmp.SubItems[ 4 ].Text;
                        while ( content.Length > 20 )
                        {
                            strBeautifulText.Add("┆" + content.Substring(0 , 20));
                            content = content.Substring(20 , content.Length - 20);
                        }
                        strBeautifulText.Add("┆" + content);
                        strBeautifulText.Add("┆————————————————————");
                        strBeautifulText.Add("┆" + tmp.SubItems[ 3 ].Text);
                        strBeautifulText.Add("╚——————————╗");
                    }
                    else
                    {

                        if ( !phoneMenber.Contains(tmp.SubItems[ 2 ].Text) )
                        {
                            phoneMenber.Add(tmp.SubItems[ 2 ].Text);
                        }
                        //四十个空格
                        string space = "                      ";
                        //是发送去的信息 就显示在右边，即多加5个制表符\t
                        strBeautifulText.Add(space + star + "【我】TO:℡﹛" + tmp.SubItems[ 2 ].Text + "﹜");
                        if ( star == "★" )
                            star = "☆";
                        else
                            star = "★";
                        strBeautifulText.Add(space + "┆————————————————————");
                        string content = tmp.SubItems[ 4 ].Text;
                        while ( content.Length > 20 )
                        {
                            strBeautifulText.Add(space + "┆" + content.Substring(0 , 20));
                            content = content.Substring(20 , content.Length - 20);
                        }
                        strBeautifulText.Add(space + "┆" + content);
                        strBeautifulText.Add(space + "┆————————————————————");
                        strBeautifulText.Add(space + "┆" + tmp.SubItems[ 3 ].Text);
                        strBeautifulText.Add("╔——————————╝");
                    }

                }

                string phoneMember = "";
                foreach ( string var in phoneMenber )
                {
                    phoneMember += "﹛" + var + "﹜";
                }
                #endregion

                strBeautifulText.Insert(3 , "┆聊天对象：" + phoneMember);
                strBeautifulText.Insert(4 , "┆聊天记录：" + lsvMessageShower.Items.Count.ToString() + "条");
                strBeautifulText.Insert(5 , "┆时间跨度：" + startTime + "　～　" + endTime);

                File.WriteAllLines(FilePath , strBeautifulText.ToArray() , Encoding.UTF8);
                return true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return false;
            }
        }

        /// <summary>
        /// 输出保存最简格式化后的文本。
        /// </summary>
        /// <param name="FilePath"></param>
        private bool YESaveTheMinText(string FilePath)
        {
            try
            {
                StringBuilder  strMinText = new StringBuilder();
                foreach ( ListViewItem tmp in lsvMessageShower.Items )
                {
                    //deliver（接收）或submit（发送）
                    if ( tmp.SubItems[ 1 ].Text != "" )
                    {

                        //是接收到的信息 就显示在左边
                        strMinText.AppendLine("{" + tmp.SubItems[ 1 ].Text + "}：" + tmp.SubItems[ 3 ].Text);
                        strMinText.AppendLine(tmp.SubItems[ 4 ].Text);
                        strMinText.AppendLine();
                    }
                    else
                    {
                        //5个空格
                        string space = "     ";
                        //是发送去的信息 就显示在右边

                        strMinText.AppendLine(space + "{" + tmp.SubItems[ 2 ].Text + "}：" + tmp.SubItems[ 3 ].Text);
                        strMinText.AppendLine(space + tmp.SubItems[ 4 ].Text);
                        strMinText.AppendLine();
                    }
                }

                File.WriteAllText(FilePath , strMinText.ToString() , Encoding.UTF8);
                return true;
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
                return false;
            }
        }

        #endregion


        #region /******************** 界面更新 代码区域 ****************************/

        /// <summary>
        /// 更新整个界面显示。
        /// </summary>
        private void YERefreshTheForm()
        {
            YETestTheEnable();
            YEResizeColumn();
            YEGetMessageType();//必须在YEGetTimeRange();函数之前，因为MyTypeCount在此函数会被重置为0；
            YEGetTimeRange();//Foreach循环1
            YEGetItemsCount();
        }

        /// <summary>
        /// 测试界面某些控件的可用性。
        /// </summary>
        private void YETestTheEnable()
        {
            //判断列表框是否含有数据，对应开启常用动作面板
            if ( lsvMessageShower.Items.Count > 0 )
            {
                grbActionBox.Enabled = true;
                grbGetWhatINeed.Enabled = true;
            }
            else
            {
                grbActionBox.Enabled = false;
                grbGetWhatINeed.Enabled = false;
            }
        }

        /// <summary>
        /// 自动调整列的宽度，达到最佳显示。
        /// </summary>
        private void YEResizeColumn()
        {
            if ( lsvMessageShower.Items.Count <= 0 )
            {
                //没有内容的时候初始化为指定宽度。
                clhType.Width = 70;
                clhSender1.Width = clhSender2.Width = clhDataTime.Width = clhContent.Width = 115;
                return;
            }
            lsvMessageShower.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        /// <summary>
        /// 获取并设置列表中包含的信息类型和种类。如飞信，QQ，Nokia短信
        /// </summary>
        private void YEGetMessageType()
        {
            try
            {
                if ( lsvMessageShower.Items.Count == 0 )
                    return;
                MyTypeCount = 0;
                lblMessageStye.Text = "";

                //测试是否有Nokia短信的类型，确定是否存在。
                if ( lsvMessageShower.FindItemWithText("sms" , true , 0) != null
                    || lsvMessageShower.FindItemWithText("mms" , true , 0) != null )
                {
                    MyTypeCount++;
                    lblMessageStye.Text += MyTypeNokia + "|";
                }
                if ( lsvMessageShower.FindItemWithText("短信" , true , 0) != null )
                {
                    MyTypeCount++;
                    lblMessageStye.Text += "短信" + "|";
                }
                if ( lsvMessageShower.FindItemWithText("彩信" , true , 0) != null )
                {
                    MyTypeCount++;
                    lblMessageStye.Text += "彩信" + "|";
                }

                //测试是否有飞信信息
                if ( lsvMessageShower.FindItemWithText(MyTypeFeiXin , true , 0) != null )
                {
                    MyTypeCount++;
                    lblMessageStye.Text += MyTypeFeiXin + "|";
                }

                //测试是否有QQ信息
                if ( lsvMessageShower.FindItemWithText(MyTypeQQ , true , 0) != null )
                {
                    MyTypeCount++;
                    lblMessageStye.Text += MyTypeQQ;
                }

                if ( lblMessageStye.Text == "" )
                    lblMessageStye.Text = "支持读取类型[QQ],[飞信],[Nokia短信]";
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
            }

        }

        /// <summary>
        /// 获取并显示当前ListView控件中的Count值
        /// </summary>
        private void YEGetItemsCount()
        {
            lblCount.Text = lsvMessageShower.Items.Count.ToString() + "条";
        }

        /// <summary>
        /// 获取ListView控件中的信息时间范围，并显示在状态栏中。
        /// </summary>
        private void YEGetTimeRange()
        {
            string time = string.Empty;
            string startTime = string.Empty;
            string endTime = string.Empty;
            //每次调用都重置此控件，保证数据的最新。
            clbDateSelected.Items.Clear();
            foreach ( ListViewItem var in lsvMessageShower.Items )
            {
                time = var.SubItems[ 3 ].Text.Substring(0 , var.SubItems[ 3 ].Text.IndexOf(' '));
                //将时间范围值都添加到clbDateSelected里面。
                if ( !clbDateSelected.Items.Contains(time) )
                    clbDateSelected.Items.Add(time);

                time = DateTime.Parse(time).ToString("yyyy-MM-dd");
                //判断最早时间和最晚时间。
                if ( startTime == "" && endTime == "" )
                    startTime = endTime = time;
                if ( time.CompareTo(startTime) < 0 )
                    startTime = time;
                if ( time.CompareTo(endTime) > 0 )
                    endTime = time;
            }
            YESetTheTimeRangeOnStatusBar(startTime , endTime);
            YESetTheTipWhenSave(startTime , endTime);
        }

        /// <summary>
        /// 把时间范围更新到状态栏。
        /// </summary>
        /// <param name="startTime"></param>
        /// <param name="endTime"></param>
        private void YESetTheTimeRangeOnStatusBar(string startTime,string endTime)
        {
            if ( startTime != "" )
                if ( startTime == endTime )
                    lblStatusInfo.Text = "[日期范围]: " + startTime;
                else
                    lblStatusInfo.Text = "[日期范围]: " + startTime + " ～ " + endTime;
            else
                lblStatusInfo.Text = "提示: 请将要读取的文件拖到到主界面区域!";
        }

        /// <summary>
        /// 把时间范围更新到“保存”对话框。
        /// </summary>
        /// <param name="startTime"></param>
        /// <param name="endTime"></param>
        private void YESetTheTipWhenSave(string startTime , string endTime)
        {
            if ( startTime == "" )
                return;

            string tips = "";
            if ( MyTypeCount == 1 )
            {
                if ( lblMessageStye.Text.Contains(MyTypeNokia) )
                {
                    if ( isThereReciver && isThereSender )
                        tips = " Nokia短信记录备份";
                    else if ( isThereSender == false )
                        tips = "收件箱";
                    else
                        tips = "发件箱";
                    
                }
                if ( lblMessageStye.Text.Contains(MyTypeFeiXin) )
                    tips = "飞信聊天记录备份";
                if ( lblMessageStye.Text.Contains(MyTypeQQ) )
                    tips = " QQ聊天记录备份";
            }
            else
                tips = "多重聊天记录备份";

            //单天的话将默认保存文件名格式推荐为：【20100101 - 】
            if ( startTime == endTime )
                MySaveFileDialog.FileName = DateTime.Parse(startTime).ToString("yyyyMMdd")
                    + " - " + tips;
            //当年份不一样的时候，直接输出两个日期。【20100101~20110101 - 】
            else if ( DateTime.Parse(startTime).ToString("yyyy") != DateTime.Parse(endTime).ToString("yyyy") )
                MySaveFileDialog.FileName = DateTime.Parse(startTime).ToString("yyyyMMdd")
                    + "～" + DateTime.Parse(endTime).ToString("yyyyMMdd")
                    + " - " + tips;
                
            //年份一样，且多天的话，将默认保存文件名格式推荐为：【20100101~0202 - 】
            else
                MySaveFileDialog.FileName = DateTime.Parse(startTime).ToString("yyyyMMdd")
                    + "～" + DateTime.Parse(endTime).ToString("MMdd")
                    + " - " + tips;
            
        }

        #endregion


        #region ======================== 自定义方法区 ==============================

        /****************************************************************/

        /// <summary>
        /// 重置整个程序。
        /// </summary>
        private void YEResetApp()
        {
            isThereSender = false;
            isThereReciver = false;
            lsvMessageShower.Items.Clear();
            lstListViewItems.Clear();
            lstListVIsame.Clear();
            lstItemsSearch.Clear();
            lstFeiXinBackup.Clear();
            ckbRemoveTheSame.Checked = false;
            //再次设置Sorting属性为None，这样就能避免了“清除”之后，再次读取文件会变超慢的问题！
            lsvMessageShower.Sorting = SortOrder.None;
        }

        /// <summary>
        /// 将反显选择的项全删除。
        /// </summary>
        private void YEDeleteSelected()
        {
            int i = 0;
            foreach ( int var in lsvMessageShower.SelectedIndices )
            {
                lsvMessageShower.Items.RemoveAt(var - i);
                ++i;
            }
            YERefreshTheForm();
        }

        /// <summary>
        /// 将勾选的项全删除。
        /// </summary>
        private void YEDeleteChecked()
        {
            int i=0;
            foreach ( int var in lsvMessageShower.CheckedIndices )
            {
                lsvMessageShower.Items.RemoveAt(var - i);
                ++i;
            }
            YERefreshTheForm();
        }

        /// <summary>
        /// 分别显示保存动作执行之后成功与否的提示对话框。
        /// </summary>
        /// <param name="isSuccess"></param>
        private void YEShowMessage(bool isSuccess)
        {
            if ( isSuccess )
                MessageBox.Show("成功保存在:\n\n" + MySaveFileDialog.FileName , @"\(^o^)/" , MessageBoxButtons.OK , MessageBoxIcon.Information);
            else
                MessageBox.Show("保存出现了错误，请重新保存！" , "-_-。sorry！" , MessageBoxButtons.OK , MessageBoxIcon.Error);
        }

        /// <summary>
        /// 自动检测短信中是否有飞信聊天记录。
        /// </summary>
        private void YEAutoCheckFeiXin()
        {
            if ( ckbAutoCheckFeixin.Checked )
            {
                lstFeiXinBackup.Clear();
                foreach ( ListViewItem tmp in lsvMessageShower.Items )
                {
                    /* 将当前状态备份为一个快照。即不想检测飞信的时候能够恢复原来的数据。
                     * 必须使用这里的Clone函数对每一项进行克隆，才能达到深度复制。否则ListViewItem作为引用类型，
                     * 使用直接赋值的话，控件里的项更改后，存在lstFeiXinBackup里面的项的值也会更改！*/
                    lstFeiXinBackup.Add((ListViewItem)tmp.Clone());

                    if ( tmp.SubItems[ 0 ].Text != "sms" )
                        continue;
                    if ( tmp.SubItems[ 1 ].Text.Contains("12520") )
                    {
                        tmp.SubItems[ 0 ].Text = "飞信";
                        tmp.SubItems[ 1 ].Text = tmp.SubItems[ 4 ].Text.Remove(tmp.SubItems[ 4 ].Text.IndexOf(':'));
                        tmp.SubItems[ 4 ].Text = tmp.SubItems[ 4 ].Text.Remove(0 , tmp.SubItems[ 4 ].Text.IndexOf(':') + 1);
                    }
                    else if ( tmp.SubItems[ 2 ].Text.Contains("12520") )
                    {
                        tmp.SubItems[ 0 ].Text = "飞信";
                        tmp.SubItems[ 2 ].Text = MyNickName.Text;//作为个人网名
                    }
                    else
                        continue;

                    //设置为飞信的时间格式。
                    tmp.SubItems[ 3 ].Text = DateTime.Parse(tmp.SubItems[ 3 ].Text).ToString("yyyy-MM-dd HH:mm:ss");
                }

            }
            else
            {
                if ( lstFeiXinBackup.Count == 0 )
                    return;
                lsvMessageShower.Items.Clear();
                lsvMessageShower.Items.AddRange(lstFeiXinBackup.ToArray());
                //马上初始化此快照。
                lstFeiXinBackup.Clear();
            }
        }

        /// <summary>
        /// 多种信息整理模式：统一时间格式，改变信息类型显示。
        /// </summary>
        private void YEMultiArrange()
        {//TODO:在此函数里面可以将手机号码显示为通讯录里面的名字。
            try
            {
                if ( ckbMultiArrange.Checked )
                {
                    lstMultiArrangeBackup.Clear();
                    foreach ( ListViewItem tmp in lsvMessageShower.Items )
                    {
                        /* 将当前状态备份为一个快照。即不想检测飞信的时候能够恢复原来的数据。
                         * 必须使用这里的Clone函数对每一项进行克隆，才能达到深度复制。否则ListViewItem作为引用类型，
                         * 使用直接赋值的话，控件里的项更改后，存在lstFeiXinBackup里面的项的值也会更改！*/
                        lstMultiArrangeBackup.Add((ListViewItem)tmp.Clone());

                        if ( tmp.SubItems[ 0 ].Text == "sms" )
                        {
                            tmp.SubItems[ 0 ].Text = "短信";
                            //设置为飞信的时间格式。
                            tmp.SubItems[ 3 ].Text = DateTime.Parse(tmp.SubItems[ 3 ].Text).ToString("yyyy-MM-dd HH:mm:ss");
                        }
                        else if ( tmp.SubItems[ 0 ].Text == "mms" )
                        {
                            tmp.SubItems[ 0 ].Text = "彩信";
                            //设置为飞信的时间格式。
                            tmp.SubItems[ 3 ].Text = DateTime.Parse(tmp.SubItems[ 3 ].Text).ToString("yyyy-MM-dd HH:mm:ss");
                        }
                        else
                            continue;

                    }

                }
                else
                {
                    if ( lstMultiArrangeBackup.Count == 0 )
                        return;
                    lsvMessageShower.Items.Clear();
                    lsvMessageShower.Items.AddRange(lstMultiArrangeBackup.ToArray());
                    //马上初始化此快照。
                    lstMultiArrangeBackup.Clear();
                }

            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
            }

        }

        /****************************************************************/

        #endregion


        #region /**************** 单击列标题实现重新排列

        #region 控件的排序比较器

        //递增排列
        class ListViewItemComparer : System.Collections.IComparer
        {
            private int col;
            public ListViewItemComparer()
            {
                col = 0;
            }
            public ListViewItemComparer(int column)
            {
                col = column;
            }
            public int Compare(object x , object y)
            {
                try
                {
                    int a = 0;
                    if ( col < ( (ListViewItem)x ).SubItems.Count && col < ( (ListViewItem)y ).SubItems.Count )
                        a = String.Compare(( (ListViewItem)x ).SubItems[ col ].Text , 
                            ((ListViewItem)y ).SubItems[ col ].Text);
                    return a;
                }
                catch ( Exception ex )
                {
                    Debug.Print("递增排序：异常\n" + ex.ToString());
                    return 0;
                }
            }
        }        

        //递减排列
        class ListViewItemComparer2 : System.Collections.IComparer
        {
            private int col;
            public ListViewItemComparer2()
            {
                col = 0;
            }
            public ListViewItemComparer2(int column)
            {
                col = column;
            }
            //更改了XY的顺序，实现相反排列。
            public int Compare(object y , object x)
            {
                try
                {
                    int a = 0;
                    if ( col < ( (ListViewItem)x ).SubItems.Count && col < ( (ListViewItem)y ).SubItems.Count )
                        a = String.Compare(( (ListViewItem)x ).SubItems[ col ].Text ,
                            ( (ListViewItem)y ).SubItems[ col ].Text);
                    return a;
                }
                catch ( Exception ex )
                {
                    Debug.Print("递减排序：异常\n" + ex.ToString());
                    return 0;
                }
            }
        }

        #endregion

        private void lsvMessageShower_ColumnClick(object sender , ColumnClickEventArgs e)
        {
            if ( lsvMessageShower.Sorting == SortOrder.Ascending )
            {
                lsvMessageShower.Sorting = SortOrder.Descending;
                lsvMessageShower.ListViewItemSorter = new ListViewItemComparer(e.Column);
            }
            else
            {
                lsvMessageShower.Sorting = SortOrder.Ascending;
                lsvMessageShower.ListViewItemSorter = new ListViewItemComparer2(e.Column);
            }
        }

        #endregion

        #region /**************** 使程序支持拖动读取文件

        //现阶段能够支持读取的文件格式
        string[] SupportedExtension = { ".txt" , ".csv" };

        private void lsvMessageShower_DragEnter(object sender , DragEventArgs e)
        {
            //TODO：将每个自定义函数假设try捕获异常块，方便跟踪所有异常！
            try
            {
                //实现手动排序的功能。
                if ( e.AllowedEffect == DragDropEffects.Move )
                    e.Effect = DragDropEffects.Move;
                else
                {
                    //马上将窗体显现出来。
                    this.TopMost = true;

                    string[] FileDrop = ( (string[])( e.Data.GetData(DataFormats.FileDrop) ) );
                    //当第一个拖动来的文件符合要求即允许拖动。
                    string extension = Path.GetExtension(FileDrop[ 0 ]);
                    if ( SupportedExtension.Contains<string>(extension) )
                        e.Effect = DragDropEffects.All;
                    else
                        e.Effect = DragDropEffects.None;

                    //最后再恢复该属性值。
                    this.TopMost = false;
                }
            }
            catch ( Exception ex )
            {
                Debug.Print(ex.ToString());
            }
        }


        private void lsvMessageShower_DragDrop(object sender , DragEventArgs e)
        {

            try
            {
                //实现手动排序的功能。
                if ( e.AllowedEffect == DragDropEffects.Move )
                {
                    int intNewIndex = 0;

                    ListViewItem movedItem = ( (ListViewItem)e.Data.GetData(lsvMessageShower.Items[ 0 ].GetType()) );
                    //必须在此单独获取坐标并转换，因为此事件先于MouseClick执行。
                    Point e2 = lsvMessageShower.PointToClient(new Point(e.X , e.Y));
                    intNewIndex = lsvMessageShower.GetItemAt(e2.X , e2.Y).Index;

                    lsvMessageShower.Items.Remove(movedItem);
                    lsvMessageShower.Items.Insert(intNewIndex , movedItem);
                }
                else
                {
                    //当拖动完成时候才开始读取文件，防止程序多次重复读取文件。

                    string[] FileDrop = ( (string[])( e.Data.GetData(DataFormats.FileDrop) ) );
                    intFileCount = FileDrop.Length;
                    foreach ( string filePath in FileDrop )
                    {
                        string extension = Path.GetExtension(filePath);
                        if ( SupportedExtension.Contains<string>(extension) )
                            YERead(filePath);
                        else
                            myToolTip.Show("此文件不支持！请重新选择！" , lsvMessageShower);
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.Print(ex.ToString());
            }
        }

        #endregion

        #region /**************** 使得 Tab 具备滑动功能

        private void myTabControl_MouseMove(object sender , MouseEventArgs e)
        {
            if ( PointToClient(MousePosition).X > myTabControl.ItemSize.Width )
                myTabControl.SelectTab(tbpOptionTab);
            else
                myTabControl.SelectTab(tbpActionTab);

        }
        
        #endregion

        #region /**************** 右键菜单 相关事件
        private void lsvMessageShower_MouseClick(object sender , MouseEventArgs e)
        {

            poiItemIndex = e.Location;

            if ( e.Button != MouseButtons.Right )
                return;

            int x1 = clhType.Width + clhSender1.Width + clhSender2.Width;
            int x2 = x1 + clhDataTime.Width;

            //当在日期区域右键点击的时候才启动。
            if ( e.X >= x1 && e.X <= x2 )
                this.tsmSelectDate.Enabled = true;
            else
                this.tsmSelectDate.Enabled = false;

        }

        private void lsvMessageShower_KeyDown(object sender , KeyEventArgs e)
        {
            if ( e.KeyCode == Keys.Delete )
                YEDeleteSelected();

            //实现全选功能。
            if ( e.Control && e.KeyCode == Keys.A )
                for ( int i=0 ; i < lsvMessageShower.Items.Count ; i++ )
                    lsvMessageShower.SelectedIndices.Add(i);
        }

        private void 删除ToolStripMenuItem_Click(object sender , EventArgs e)
        {
            YEDeleteSelected();
            myContextMenuStrip.Hide();
        }
        private void 勾选的ToolStripMenuItem_Click(object sender , EventArgs e)
        {
            YEDeleteChecked();
        }

        private void tsmSelectAll_Click(object sender , EventArgs e)
        {
            //实现全选功能。
            for ( int i=0 ; i < lsvMessageShower.Items.Count ; i++ )
                lsvMessageShower.SelectedIndices.Add(i);
        }

        private void tsmSelectOthers_Click(object sender , EventArgs e)
        {
            //实现反选功能.
            for ( int i=0 ; i < lsvMessageShower.Items.Count ; i++ )
            {
                if ( lsvMessageShower.SelectedIndices.Contains(i) )
                    lsvMessageShower.SelectedIndices.Remove(i);
                else
                    lsvMessageShower.SelectedIndices.Add(i);
            }

        }

        //把选择的下级菜单中的“日期”变为：“2010-01-01的信息”
        private void myContextMenuStrip_Opening(object sender , CancelEventArgs e)
        {
            if ( lsvMessageShower.Items.Count == 0 )
                return;
            string  time = lsvMessageShower.SelectedItems[ 0 ].SubItems[ 3 ].Text.Substring(0 ,
                lsvMessageShower.SelectedItems[ 0 ].SubItems[ 3 ].Text.IndexOf(' '));
            tsmSelectDate.Text = time + "的信息";
        }
        
        //选取当前选择项相同时间的信息。
        private void tsmSelectDate_Click(object sender , EventArgs e)
        {
            if ( tsmSelectDate.Text == "日期(&D)" )
                return;
            clbDateSelected.SetItemChecked(clbDateSelected.Items.IndexOf(tsmSelectDate.Text.Remove(10)),true);
            YEGetIt();
        }
        #endregion

        #region/**************** 选择常用动作框 相关事件*********************/
        private void btnOpen_Click(object sender , EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "逗号分隔值文件(*.csv)|*.csv|文本文件(*.txt)|*.txt|所有文件(*.*)|*.*";
            ofd.Multiselect = true;
            if ( ofd.ShowDialog(this) == DialogResult.OK )
            {
                intFileCount = ofd.FileNames.Length;
                foreach ( string filePath in ofd.FileNames )
                {
                    string extension = Path.GetExtension(filePath);
                    if ( SupportedExtension.Contains<string>(extension) )
                        YERead(filePath);
                    else
                        myToolTip.Show("此文件不支持！请重新选择！" ,lsvMessageShower);
                }
            }
        }

        private void button1_Click(object sender , EventArgs e)
        {
            Process.Start(Application.StartupPath);
        }

        private void btnClear_Click(object sender , EventArgs e)
        {
            YEResetApp();
            YERefreshTheForm();
        }


        private void btnClose_Click(object sender , EventArgs e)
        {
            this.Close();
        }
        #endregion

        #region/**************** 选择输出动作框 相关事件*********************/
        private void btnSaveTheMin_Click(object sender , EventArgs e)
        {

            //将顺序默认排列为：接收在前，日期递增排列。
            lsvMessageShower.ListViewItemSorter = new ListViewItemComparer(0);
            lsvMessageShower.ListViewItemSorter = new ListViewItemComparer(3);

            MySaveFileDialog.Filter = "文本文件(*.txt)|*.txt";
            MySaveFileDialog.FileName += "の最简格式化显示";
            if ( MySaveFileDialog.ShowDialog(this) == DialogResult.OK )
                YEShowMessage(YESaveTheMinText(MySaveFileDialog.FileName));
        }

        private void btnSaveBeautifulText_Click(object sender , EventArgs e)
        {
            //将顺序默认排列为：接收在前，日期递增排列。
            lsvMessageShower.ListViewItemSorter = new ListViewItemComparer(0);
            lsvMessageShower.ListViewItemSorter = new ListViewItemComparer(3);

            MySaveFileDialog.Filter = "文本文件(*.txt)|*.txt";
            MySaveFileDialog.FileName += "の格式后美化显示";
            if ( MySaveFileDialog.ShowDialog(this) == DialogResult.OK )
                YEShowMessage(YESaveBeautifulText(MySaveFileDialog.FileName));
        }

        private void btnSave_Click(object sender , EventArgs e)
        {
            MySaveFileDialog.Filter = "文本文件(*.txt)|*.txt|逗号分隔值文件(*.csv)|*.csv";
            if ( MySaveFileDialog.ShowDialog(this) == DialogResult.OK )
                if ( lblMessageStye.Text.Contains(MyTypeNokia) && MyTypeCount == 1 )
                    YEShowMessage(YESaveNokiaMessages(MySaveFileDialog.FileName));
                else
                    YEShowMessage(YESaveNewList(MySaveFileDialog.FileName));
        }
        #endregion

        #region/**************** 截取指定信息框 相关事件*********************/
        private void btnGetIt_Click(object sender , EventArgs e)
        {
            YEGetIt();
        }

        private void YEGetIt()
        {

            //重置该变量
            lstListVIsame.Clear();
            ckbRemoveTheSame.Checked = false;

            //首先将当前ListView控件里面的Items数据保存一个副本。
            //然后在这个副本里面查找需要的信息并显示（仅对返回的数据显示，不涉及修改它本身，因此它还是一个完整的状态）
            //随后可以随时恢复控件之前的状态。

            if ( lstListViewItems.Count == 0 )
                lstListViewItems.AddRange(lsvMessageShower.Items.Cast<ListViewItem>());

            lsvMessageShower.Items.Clear();

            if ( clbDateSelected.CheckedItems.Count == 0 )
            {
                lsvMessageShower.Items.AddRange(lstListViewItems.ToArray());
                lstListViewItems.Clear();
            }
            else
                lsvMessageShower.Items.AddRange(lstListViewItems.FindAll(YEFindWhatINeed).ToArray());

            YERefreshTheForm();
        }

        private bool YEFindWhatINeed(ListViewItem lvi)
        {
            foreach ( Object var in clbDateSelected.CheckedItems )
                if ( lvi.SubItems[ 3 ].Text.Contains(var.ToString()) )
                    return true;
            return false;
        }
        #endregion

        #region/**************** 自动去除相同信息 相关事件*******************/
        private void ckbRemoveTheSame_CheckedChanged(object sender , EventArgs e)
        {
            YERemoveTheSame();
        }

        private void YERemoveTheSame()
        {
            //总是将当前列表框首次最完整的状态保存到lstListViewItems中。
            if ( lstListViewItems.Count == 0 )
                lstListViewItems.AddRange(lsvMessageShower.Items.Cast<ListViewItem>());

            //将当前最新状态保存到lstListVIsame中。
            if ( ckbRemoveTheSame.Checked )
            {
                lstListVIsame.Clear();
                lstListVIsame.AddRange(lsvMessageShower.Items.Cast<ListViewItem>());
                lsvMessageShower.Items.Clear();
                //【剔除重复信息的代码】
                lsvMessageShower.Items.AddRange(lstListVIsame.Distinct<ListViewItem>(new DistinctComparer()).ToArray());
            }
            else 
            {
                lsvMessageShower.Items.Clear();
                lsvMessageShower.Items.AddRange(lstListVIsame.ToArray());
            }
            //当lstListVIsame被初始化之时，不要更新界面，那样会造成选取动作无法执行。
            if ( lstListVIsame.Count != 0 )
                YERefreshTheForm();
        }

        //用于剔除重复信息的比较器。
        class DistinctComparer : IEqualityComparer<ListViewItem>
        {
            #region IEqualityComparer<ListViewItem> 成员
            public bool Equals(ListViewItem x , ListViewItem y)
            {
                if ( x.SubItems[ 0 ].Text == y.SubItems[ 0 ].Text
                    && x.SubItems[ 1 ].Text == y.SubItems[ 1 ].Text
                    && x.SubItems[ 2 ].Text == y.SubItems[ 2 ].Text
                    && x.SubItems[ 3 ].Text == y.SubItems[ 3 ].Text
                    && x.SubItems[ 4 ].Text == y.SubItems[ 4 ].Text
                    )
                    return true;
                return false;
            }

            public int GetHashCode(ListViewItem obj)
            {
                //必须返回ToString值的hashCode才能实现相同剔除。
                //只要返回的时间，内容的HashCode相同，则百分之99以上是相同的了。
                //这样Equals的调用量就会少很多，只是增加了此GetHashCode的计算量。
                return obj.SubItems[ 3 ].Text.GetHashCode()+obj.SubItems[ 4 ].Text.GetHashCode();
            }
            #endregion
        }

        #endregion

        #region/**************** 搜索指定信息 相关事件*******************/
        private void YESearch()
        {
            //将信息添加到下拉框
            if ( !cbbSearchText.Items.Contains(cbbSearchText.Text) )
            {
                cbbSearchText.Items.Add(cbbSearchText.Text);
                cbbSearchText.AutoCompleteCustomSource.Add(cbbSearchText.Text);
            }

            if ( lstItemsSearch.Count == 0 )
                lstItemsSearch.AddRange(lsvMessageShower.Items.Cast<ListViewItem>());

            lsvMessageShower.Items.Clear();

            if ( cbbSearchText.Text != "" )
                lsvMessageShower.Items.AddRange(lstItemsSearch.FindAll(YESearchWhatINeed).ToArray());
            else
            {
                lsvMessageShower.Items.AddRange(lstItemsSearch.ToArray());
                lstItemsSearch.Clear();
            }

            YERefreshTheForm();
        }

        private bool YESearchWhatINeed(ListViewItem lsv)
        {
            if ( lsv.SubItems[ 0 ].Text.Contains(cbbSearchText.Text)
                || lsv.SubItems[ 1 ].Text.Contains(cbbSearchText.Text)
                || lsv.SubItems[ 2 ].Text.Contains(cbbSearchText.Text)
                || lsv.SubItems[ 3 ].Text.Contains(cbbSearchText.Text)
                || lsv.SubItems[ 4 ].Text.Contains(cbbSearchText.Text) )
                return true;
            return false;
        }

        private void btnSearch_Click(object sender , EventArgs e)
        {
            YESearch();
        }

        private void cbbSearchText_KeyDown(object sender , KeyEventArgs e)
        {
            if ( e.KeyCode == Keys.Enter )
                YESearch();
        }
        #endregion

        public frmChattingHistoryManager()
        {
            InitializeComponent();
        }

        private void frmChattingHistoryManager_SizeChanged(object sender , EventArgs e)
        {
            YEResizeColumn();
        }

        private void linkLabel1_LinkClicked(object sender , LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                Process.Start("http://asion.cnblogs.com");
            }
            catch(Exception ex)
            {
                Debug.Print(ex.ToString());
            }
        }

        private void ckbAutoCheckFeixin_CheckedChanged(object sender , EventArgs e)
        {
            //Checked属性更改的时候就默认自动刷新上一次打开的文件。
            YEAutoCheckFeiXin();
            YERefreshTheForm();
        }

        private void ckbMultiArrange_CheckedChanged(object sender , EventArgs e)
        {
            //默认启动“自动检测飞信的功能”
            ckbAutoCheckFeixin.Checked = true;
            YEMultiArrange();
            YERefreshTheForm();
        }

        private void lsvMessageShower_ItemDrag(object sender , ItemDragEventArgs e)
        {
            //将列表框的排序功能关闭
            lsvMessageShower.Sorting = SortOrder.None;
            //能够实现手动拖动排序功能。
            if ( e.Button == MouseButtons.Left )
                lsvMessageShower.DoDragDrop(e.Item , DragDropEffects.Move);

        }

        private void tsmCopyInfo_Click(object sender , EventArgs e)
        {
            //能够复制发送人的名字，方便设置“我的网名”。
           ListViewItem tmp = lsvMessageShower.GetItemAt(poiItemIndex.X , poiItemIndex.Y);
           if ( tmp != null )
               MyNickName.Text = ( tmp.SubItems[ 1 ].Text != "" ) ? tmp.SubItems[ 1 ].Text : tmp.SubItems[ 2 ].Text;
        }

    }
}
